var rect = document.querySelector('rect#map');
var toFixed = n => Number.parseFloat(n).toFixed(2);
var radToDeg = radians => radians * 180 / Math.PI;

var svgns = "http://www.w3.org/2000/svg",
    svg = document.querySelector('svg');

rect.addEventListener('mousemove', e => {
  var rect = e.target.getBoundingClientRect();
  var x = e.clientX - rect.left; // x position within the element
  var y = e.clientY - rect.top;  // y position within the element

  // console.log(
  //   'x: ' + toFixed(x / rect.width * e.target.width.baseVal.valueInSpecifiedUnits) + '"',
  //   'y: ' + toFixed(y / rect.height * e.target.height.baseVal.valueInSpecifiedUnits) + '"'
  // );

  let aim = document.querySelector('.firing-arc.active');

  if (aim) {
    aim.setAttributeNS(null, 'x2', `${x / rect.width * e.target.width.baseVal.valueInSpecifiedUnits}in`);
    aim.setAttributeNS(null, 'y2', `${y / rect.height * e.target.height.baseVal.valueInSpecifiedUnits}in`);

    let w = aim.x2.baseVal.value - aim.x1.baseVal.value;
    let h = -(aim.y2.baseVal.value - aim.y1.baseVal.value);
    let angle = radToDeg(Math.atan(h / w));

    if (w < 0 && h > 0) {
      angle = 180 - Math.abs(angle);
    } else if (w < 0 && h < 0) {
      angle = 180 + Math.abs(angle);
    } else if (w > 0 && h < 0) {
      angle = 360 - Math.abs(angle);
    }

    console.log(`${toFixed(angle)}\u00B0`);

    let [x1, y1, x2, y2] = [aim.x1, aim.y1, aim.x2, aim.y2].map(v => v.baseVal.valueInSpecifiedUnits);
    let [width, height] = [e.target.width, e.target.height].map(v => v.baseVal.valueInSpecifiedUnits);

    console.log(`(${x2 - x1}, ${y2 - y1})`);

    if (x2 - x1 > 0 && y2 - y1 > 0) {
      let yWhenXisMax = (width - x1) * (y2 - y1) / (x2 - x1) + y1;
      let xWhenYisMax = (height - y1) * (x2 - x1) / (y2 - y1) + x1;

      if (xWhenYisMax <= width) {
        aim.setAttributeNS(null, 'x2', `${xWhenYisMax}in`);
        aim.setAttributeNS(null, 'y2', `${height}in`);
      } else {
        aim.setAttributeNS(null, 'x2', `${width}in`);
        aim.setAttributeNS(null, 'y2', `${yWhenXisMax}in`);
      }
    } else if (x2 - x1 > 0 && y2 - y1 < 0) {
      let yWhenXisMax = (width - x1) * (y2 - y1) / (x2 - x1) + y1;
      let xWhenYisZero = (0 - y1) * (x2 - x1) / (y2 - y1) + x1;

      if (xWhenYisZero <= width) {
        aim.setAttributeNS(null, 'x2', `${xWhenYisZero}in`);
        aim.setAttributeNS(null, 'y2', `0`);
      } else {
        aim.setAttributeNS(null, 'x2', `${width}in`);
        aim.setAttributeNS(null, 'y2', `${yWhenXisMax}in`);
      }
    } else if (x2 - x1 < 0 && y2 - y1 < 0) {
      let yWhenXisZero = (0 - x1) * (y2 - y1) / (x2 - x1) + y1;
      let xWhenYisZero = (0 - y1) * (x2 - x1) / (y2 - y1) + x1;

      if (yWhenXisZero >= 0) {
        aim.setAttributeNS(null, 'x2', `0`);
        aim.setAttributeNS(null, 'y2', `${yWhenXisZero}in`);
      } else {
        aim.setAttributeNS(null, 'x2', `${xWhenYisZero}in`);
        aim.setAttributeNS(null, 'y2', `0`);
      }
    } else {
      let yWhenXisZero = (0 - x1) * (y2 - y1) / (x2 - x1) + y1;
      let xWhenYisMax = (height - y1) * (x2 - x1) / (y2 - y1) + x1;

      if (yWhenXisZero <= height) {
        aim.setAttributeNS(null, 'x2', `0`);
        aim.setAttributeNS(null, 'y2', `${yWhenXisZero}in`);
      } else {
        aim.setAttributeNS(null, 'x2', `${xWhenYisMax}in`);
        aim.setAttributeNS(null, 'y2', `${height}in`);
      }
    }
  }
});

document.querySelectorAll('.soldier-record').forEach(el =>
  el.addEventListener('click', e => {
    if (e.target.classList.contains('selected')) {
      e.target.classList.remove('selected');
    } else {
      document.querySelectorAll('.soldier-record.selected').forEach(el =>
        el.classList.remove('selected')
      );
      e.target.classList.add('selected');
    }
  })
);

document.querySelector('#set-firing-arc').addEventListener('click', e => {
  let selectedSoldier = document.querySelector('.soldier-record.selected');

  if (selectedSoldier) {
    let {troopNumber, troopAllegiance} = selectedSoldier.dataset;

    let firingArc = document.querySelector(
      `.firing-arc[data-troop-number="${troopNumber}"][data-troop-allegiance="${troopAllegiance}"]`
    );

    if (firingArc) { firingArc.remove() }

    let counter = document.querySelector(
      `circle.counter[data-troop-number="${troopNumber}"][data-troop-allegiance="${troopAllegiance}"]`
    );

    let aim = document.createElementNS(svgns, 'line');

    aim.classList.add('firing-arc', 'active');
    aim.dataset.troopNumber = troopNumber;
    aim.dataset.troopAllegiance = troopAllegiance;
    aim.setAttributeNS(null, 'x1', counter.cx.baseVal.valueAsString);
    aim.setAttributeNS(null, 'y1', counter.cy.baseVal.valueAsString);
    aim.setAttributeNS(null, 'x2', counter.cx.baseVal.valueAsString);
    aim.setAttributeNS(null, 'y2', counter.cy.baseVal.valueAsString);
    // aim.style.transformOrigin = `${counter.cx.baseVal.valueAsString} ${counter.cy.baseVal.valueAsString}`;

    svg.appendChild(aim);

    aim.addEventListener('click', e => {
      e.target.classList.remove('active');
      document.querySelector('circle#point').style.display = '';
    });

    document.querySelector('circle#point').style.display = 'none';
  }
});

var columnCount = 33,
    rowCount = 25,
    pointDistanceInInches = 1.005;

var isEven = n => n % 2 === 0;

var columns = [...Array(columnCount).keys()],
    rows = [...Array(rowCount).keys()],
    points = rows.map(y => columns.map(x => [x, y]));

var xOffset = 0.25,
    yOffset = 0.45;
    calcY = Math.sqrt(3) * pointDistanceInInches / 2,
    alternatingOffset = pointDistanceInInches / 2;

points.forEach((row, index) => row.forEach(([x, y]) => {
  var cx = x * pointDistanceInInches + xOffset + (isEven(index) ? alternatingOffset : 0),
      cy = y * pointDistanceInInches * calcY + yOffset,
      point = document.createElementNS(svgns, 'use');

  point.setAttributeNS(null, 'href', `#point`);
  point.setAttributeNS(null, 'x', `${cx}in`);
  point.setAttributeNS(null, 'y', `${cy}in`);
  point.dataset.x = x;
  point.dataset.y = y;

  point.addEventListener('click', e => {
    let selectedSoldier = document.querySelector('.soldier-record.selected');

    if (selectedSoldier) {
      let counter = document.createElementNS(svgns, 'circle');
      let text = document.createElementNS(svgns, 'text');

      counter.setAttributeNS(null, 'cx', `${cx}in`);
      counter.setAttributeNS(null, 'cy', `${cy}in`);
      counter.setAttributeNS(null, 'r', '0.25in');
      counter.dataset.troopNumber = selectedSoldier.dataset.troopNumber;
      counter.dataset.troopAllegiance = selectedSoldier.dataset.troopAllegiance;
      counter.classList.add('counter');

      text.setAttributeNS(null, 'text-anchor', 'middle');
      text.setAttributeNS(null, 'x', `${cx}in`);
      text.setAttributeNS(null, 'y', `${cy + 0.25}in`);
      text.dataset.troopNumber = selectedSoldier.dataset.troopNumber;
      text.dataset.troopAllegiance = selectedSoldier.dataset.troopAllegiance;
      text.textContent = `${selectedSoldier.dataset.troopNumber}`;
      text.classList.add('counter');

      document.querySelectorAll(
        `.counter[data-troop-number="${selectedSoldier.dataset.troopNumber}"][data-troop-allegiance="${selectedSoldier.dataset.troopAllegiance}"]`
      ).forEach(el => el.remove());

      svg.appendChild(counter);
      svg.appendChild(text);
    }
  });

  svg.appendChild(point);
}));

let getPointCoords = (x, y) => {
  let point = document.querySelector(`[data-x="${x}"][data-y="${y}"]`);

  return [point.x.baseVal.value, point.y.baseVal.value]
};


let smallArc = document.createElementNS(svgns, 'polygon');
smallArc.setAttributeNS(null, 'id', 'small-arc');
let smallArcPoints = `${getPointCoords(1, 24)} ${getPointCoords(1, 21)} ${getPointCoords(2, 21)}`;
smallArc.setAttributeNS(null, 'points', smallArcPoints);

svg.appendChild(smallArc);

let medArc = document.createElementNS(svgns, 'polygon');
medArc.setAttributeNS(null, 'id', 'med-arc');
let medArcPoints = `${getPointCoords(4, 24)} ${getPointCoords(3, 21)} ${getPointCoords(6, 21)}`;
medArc.setAttributeNS(null, 'points', medArcPoints);

svg.appendChild(medArc);

let lrgArc = document.createElementNS(svgns, 'polygon');
lrgArc.setAttributeNS(null, 'id', 'lrg-arc');
let lrgArcPoints = `${getPointCoords(19, 24)} ${getPointCoords(9, 21)} ${getPointCoords(30, 21)}`;
lrgArc.setAttributeNS(null, 'points', lrgArcPoints);

svg.appendChild(lrgArc);

arc = document.querySelector('#med-arc');
let [ox, oy] = getPointCoords(4, 24);
arc.style.transformOrigin = `${ox}px ${oy}px`;
arc.style.transform = 'rotate(90deg)';